package cn.dg32z.lon.checks.util.processor.rotateprocessor;

import cn.dg32z.libs.com.github.retrooper.packetevents.event.PacketReceiveEvent;
import cn.dg32z.libs.com.github.retrooper.packetevents.wrapper.play.client.WrapperPlayClientPlayerFlying;
import cn.dg32z.lon.PluginLoader;
import cn.dg32z.lon.checks.Check;
import cn.dg32z.lon.checks.impl.combat.analysis.AnalysisUtils;
import cn.dg32z.lon.checks.type.PacketCheck;
import cn.dg32z.lon.checks.type.RotationCheck;
import cn.dg32z.lon.player.PlayerData;
import cn.dg32z.lon.utils.data.HeadRotation;
import cn.dg32z.lon.utils.graphing.GraphUtil;
import cn.dg32z.lon.utils.math.MathUtil;
import cn.dg32z.lon.utils.update.RotationUpdate;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/* loaded from: cn_dg32z_lon_checks_util_processor_rotateprocessor.jar:cn/dg32z/lon/checks/util/processor/rotateprocessor/Cinematic.class */
public final class Cinematic extends Check implements RotationCheck, PacketCheck {
    private static final double CINEMATIC_CONSTANT = 0.0078125d;
    private final List<Double> yawSamples;
    private final List<Double> pitchSamples;
    private int isTotallyNotCinematic;
    private long lastSmooth;
    private long lastHighRate;
    private int lastCinematicTicks;
    private int cinematicTicks;
    private int tick;

    public Cinematic(PlayerData player) {
        super(player);
        this.yawSamples = new ArrayList();
        this.pitchSamples = new ArrayList();
        this.isTotallyNotCinematic = 0;
        this.lastSmooth = 0L;
        this.lastHighRate = 0L;
    }

    public void onPacketReceive(PacketReceiveEvent event) {
        if (WrapperPlayClientPlayerFlying.isFlying(event.getPacketType())) {
            this.tick++;
        } else {
            this.tick = 0;
        }
    }

    public void process(RotationUpdate rotationUpdate) {
        HeadRotation from = rotationUpdate.getFrom();
        HeadRotation to = rotationUpdate.getTo();
        long now = time();
        float deltaYaw = Math.abs(to.getYaw() - from.getYaw());
        float deltaPitch = Math.abs(to.getPitch() - from.getPitch());
        float lastDeltaYaw = rotationUpdate.getProcessor().getLastLastYaw();
        float lastDeltaPitch = rotationUpdate.getProcessor().getLastLastPitch();
        double differenceYaw = Math.abs(deltaYaw - lastDeltaYaw);
        double differencePitch = Math.abs(deltaPitch - lastDeltaPitch);
        double joltYaw = Math.abs(differenceYaw - deltaYaw);
        double joltPitch = Math.abs(differencePitch - deltaPitch);
        boolean cinematic2 = now - this.lastHighRate > 250 || now - this.lastSmooth < 9000;
        if (joltYaw > 1.0d && joltPitch > 1.0d) {
            this.lastHighRate = now;
        }
        if (deltaYaw > 0.0f && deltaPitch > 0.0f) {
            this.yawSamples.add(Double.valueOf(deltaYaw));
            this.pitchSamples.add(Double.valueOf(deltaPitch));
            if (this.yawSamples.size() >= 20 && this.pitchSamples.size() >= 20) {
                Set<Double> shannonYaw = new HashSet<>();
                Set<Double> shannonPitch = new HashSet<>();
                List<Double> stackYaw = new ArrayList<>();
                List<Double> stackPitch = new ArrayList<>();
                for (Double yawSample : this.yawSamples) {
                    stackYaw.add(yawSample);
                    stackPitch.add(yawSample);
                    if (stackYaw.size() >= 10 && stackPitch.size() >= 10) {
                        shannonYaw.add(Double.valueOf(AnalysisUtils.getShannonEntropy(stackYaw)));
                        shannonPitch.add(Double.valueOf(AnalysisUtils.getShannonEntropy(stackPitch)));
                        stackYaw.clear();
                        stackPitch.clear();
                    }
                }
                if (shannonYaw.size() != 1 || shannonPitch.size() != 1 || !shannonYaw.iterator().next().equals(shannonPitch.iterator().next())) {
                    this.isTotallyNotCinematic = 20;
                }
                GraphUtil.GraphResult resultsYaw = GraphUtil.getGraph(this.yawSamples);
                GraphUtil.GraphResult resultsPitch = GraphUtil.getGraph(this.pitchSamples);
                if (resultsYaw.positives() > resultsYaw.negatives() || resultsPitch.positives() > resultsPitch.negatives()) {
                    this.lastSmooth = now;
                }
                this.yawSamples.clear();
                this.pitchSamples.clear();
            }
            updateCinematic2(rotationUpdate, cinematic2);
        }
        updateCinematic(rotationUpdate);
    }

    private void updateCinematic2(RotationUpdate rotationUpdate, boolean cinematic2Flag) {
        boolean allowed = PluginLoader.INSTANCE.getConfigManager().getConfig().getBooleanElse("function.allowed-cinematic", false);
        if (this.isTotallyNotCinematic > 0 || !allowed) {
            this.isTotallyNotCinematic--;
            rotationUpdate.setCinematic2(false);
        } else {
            rotationUpdate.setCinematic2(cinematic2Flag);
        }
    }

    private void updateCinematic(RotationUpdate rotationUpdate) {
        float deltaPitch = rotationUpdate.getProcessor().getDeltaPitch();
        float deltaYaw = rotationUpdate.getProcessor().getDeltaYaw();
        if (deltaPitch == 0.0f || deltaYaw == 0.0f) {
            return;
        }
        float yawAccel = rotationUpdate.getProcessor().getYawAccel();
        float pitchAccel = rotationUpdate.getProcessor().getPitchAccel();
        float lastDeltaYaw = rotationUpdate.getProcessor().getLastDeltaYaw();
        float lastDeltaPitch = rotationUpdate.getProcessor().getLastDeltaPitch();
        boolean invalid = MathUtil.isExponentiallySmall(Float.valueOf(yawAccel)) || yawAccel == 0.0f || MathUtil.isExponentiallySmall(Float.valueOf(pitchAccel)) || pitchAccel == 0.0f;
        long expandedDeltaYaw = (long) (deltaYaw * MathUtil.EXPANDER);
        long expandedLastDeltaYaw = (long) (lastDeltaYaw * MathUtil.EXPANDER);
        long expandedDeltaPitch = (long) (deltaPitch * MathUtil.EXPANDER);
        long expandedLastDeltaPitch = (long) (lastDeltaPitch * MathUtil.EXPANDER);
        double constantYaw = MathUtil.getGcd(expandedDeltaYaw, expandedLastDeltaYaw);
        double constantPitch = MathUtil.getGcd(expandedDeltaPitch, expandedLastDeltaPitch);
        boolean cinematic = !invalid && yawAccel < 1.0f && pitchAccel < 1.0f;
        if (!cinematic) {
            this.cinematicTicks = Math.max(this.cinematicTicks - 1, 0);
        } else if (constantYaw < 0.0078125d && constantPitch < 0.0078125d) {
            this.cinematicTicks++;
        }
        if (this.cinematicTicks > 5) {
            this.cinematicTicks--;
        }
        boolean allowed = PluginLoader.INSTANCE.getConfigManager().getConfig().getBooleanElse("function.allowed-cinematic", false);
        rotationUpdate.setCinematic((this.cinematicTicks > 2 || this.tick - this.lastCinematicTicks < 80) && allowed);
        if (rotationUpdate.isCinematic() && this.cinematicTicks > 3) {
            this.lastCinematicTicks = this.tick;
        }
    }
}
